package java.security;

import java.io.IOException;
import java.security.spec.InvalidParameterSpecException;
import java.security.spec.AlgorithmParameterSpec;

public class AlgorithmParameters
{
    private Provider provider;
    private AlgorithmParametersSpi paramSpi;
    private String algorithm;
    private boolean initialized;
    
    protected AlgorithmParameters(AlgorithmParametersSpi paramSpi, Provider provider, String algorithm) {
        this.initialized = false;
        this.paramSpi = paramSpi;
        this.provider = provider;
        this.algorithm = algorithm;
    }
    
    public final String getAlgorithm() {
        return this.algorithm;
    }
    
    public static AlgorithmParameters getInstance(String str) throws NoSuchAlgorithmException {
        try {
            Object[] impl = Security.getImpl(str, "AlgorithmParameters", (String)null);
            return new AlgorithmParameters((AlgorithmParametersSpi)impl[0], (Provider)impl[1], str);
        }
        catch (NoSuchProviderException ex) {
            throw new NoSuchAlgorithmException(str + " not found");
        }
    }
    
    public static AlgorithmParameters getInstance(String s, String s2) throws NoSuchAlgorithmException, NoSuchProviderException {
        if (s2 == null || s2.length() == 0) {
            throw new IllegalArgumentException("missing provider");
        }
        Object[] impl = Security.getImpl(s, "AlgorithmParameters", s2);
        return new AlgorithmParameters((AlgorithmParametersSpi)impl[0], (Provider)impl[1], s);
    }
    
    public static AlgorithmParameters getInstance(String s, Provider provider) throws NoSuchAlgorithmException {
        if (provider == null) {
            throw new IllegalArgumentException("missing provider");
        }
        Object[] impl = Security.getImpl(s, "AlgorithmParameters", provider);
        return new AlgorithmParameters((AlgorithmParametersSpi)impl[0], (Provider)impl[1], s);
    }
    
    public final Provider getProvider() {
        return this.provider;
    }
    
    public final void init(AlgorithmParameterSpec algorithmParameterSpec) throws InvalidParameterSpecException {
        if (this.initialized) {
            throw new InvalidParameterSpecException("already initialized");
        }
        this.paramSpi.engineInit(algorithmParameterSpec);
        this.initialized = true;
        return;
    }
    
    public final void init(byte[] array) throws IOException {
        if (this.initialized) {
            throw new IOException("already initialized");
        }
        this.paramSpi.engineInit(array);
        this.initialized = true;
        return;
    }
    
    public final void init(byte[] array, String s) throws IOException {
        if (this.initialized) {
            throw new IOException("already initialized");
        }
        this.paramSpi.engineInit(array, s);
        this.initialized = true;
        return;
    }
    
    public final <T extends AlgorithmParameterSpec> T getParameterSpec(Class<T> clazz) throws InvalidParameterSpecException {
        if (!this.initialized) {
            throw new InvalidParameterSpecException("not initialized");
        }
        return this.paramSpi.engineGetParameterSpec(clazz);
    }
    
    public final byte[] getEncoded() throws IOException {
        if (!this.initialized) {
            throw new IOException("not initialized");
        }
        return this.paramSpi.engineGetEncoded();
    }
    
    public final byte[] getEncoded(String s) throws IOException {
        if (!this.initialized) {
            throw new IOException("not initialized");
        }
        return this.paramSpi.engineGetEncoded(s);
    }
    
    @Override
    public final String toString() {
        if (!this.initialized) {
            return null;
        }
        return this.paramSpi.engineToString();
    }
}
